import { Driver } from "./Driver";
import { SchemaBuilder } from "../schema-builder/SchemaBuilder";
import { BaseDriver } from "./BaseDriver";
import { DriverOptions } from "./DriverOptions";
import { ObjectLiteral } from "../common/ObjectLiteral";
import { DatabaseConnection } from "./DatabaseConnection";
/**
 * This driver organizes work with mysql database.
 */
export declare class MysqlDriver extends BaseDriver implements Driver {
    connectionOptions: DriverOptions;
    readonly isResultsLowercase: boolean;
    /**
     * Mysql library.
     */
    private mysql;
    /**
     * Connection to mysql database.
     */
    private mysqlConnection;
    /**
     * Connection to mysql database.
     */
    private mysqlPool;
    private databaseConnectionPool;
    retrieveDatabaseConnection(): Promise<DatabaseConnection>;
    releaseDatabaseConnection(dbConnection: DatabaseConnection): Promise<void>;
    /**
     * Access to the native implementation of the database.
     */
    readonly native: {
        driver: any;
        connection: DatabaseConnection | undefined;
        pool: any;
    };
    /**
     * Access to the native connection to the database.
     */
    readonly nativeConnection: any;
    /**
     * Database name to which this connection is made.
     */
    readonly db: string;
    constructor(connectionOptions: DriverOptions, mysql?: any);
    /**
     * Creates a schema builder which can be used to build database/table schemas.
     */
    createSchemaBuilder(dbConnection: DatabaseConnection): SchemaBuilder;
    /**
     * Performs connection to the database based on given connection options.
     */
    connect(): Promise<void>;
    /**
     * Closes connection with database.
     */
    disconnect(): Promise<void>;
    /**
     * Escapes given value.
     */
    escape(dbConnection: DatabaseConnection, value: any): any;
    /**
     * Executes a given SQL query.
     */
    query<T>(dbConnection: DatabaseConnection, query: string, parameters?: any[]): Promise<T>;
    /**
     * Starts transaction.
     */
    beginTransaction(dbConnection: DatabaseConnection): Promise<void>;
    /**
     * Commits transaction.
     */
    commitTransaction(dbConnection: DatabaseConnection): Promise<void>;
    /**
     * Rollbacks transaction.
     */
    rollbackTransaction(dbConnection: DatabaseConnection): Promise<void>;
    /**
     * Clears all tables in the currently connected database.
     */
    clearDatabase(dbConnection: DatabaseConnection): Promise<void>;
    buildParameters(sql: string, parameters: ObjectLiteral): any[];
    replaceParameters(sql: string, parameters: ObjectLiteral): string;
    protected checkIfConnectionSet(): void;
}
