"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const SchemaBuilder_1 = require("./SchemaBuilder");
const ForeignKeyMetadata_1 = require("../metadata/ForeignKeyMetadata");
/**
 * @internal
 */
class PostgresSchemaBuilder extends SchemaBuilder_1.SchemaBuilder {
    constructor(driver, dbConnection) {
        super();
        this.driver = driver;
        this.dbConnection = dbConnection;
    }
    /*async getColumnProperties(tableName: string, columnName: string): Promise<{ isNullable: boolean, columnType: string, autoIncrement: boolean }|undefined> {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_CATALOG = '${this.driver.db}'` +
            ` AND TABLE_NAME = '${tableName}' AND COLUMN_NAME = '${columnName}'`;

        const result = await this.query<ObjectLiteral>(sql);
        return {
            isNullable: result["IS_NULLABLE"] === "YES",
            columnType: result["COLUMN_TYPE"],
            autoIncrement: result["EXTRA"].indexOf("auto_increment") !== -1
        };
    }*/
    getChangedColumns(tableName, columns) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_CATALOG = '${this.driver.db}'` +
            ` AND TABLE_NAME = '${tableName}'`;
        return this.query(sql).then(results => {
            // console.log("changed columns: ", results);
            return columns.filter(column => {
                const dbData = results.find(result => result.column_name === column.name);
                if (!dbData)
                    return false;
                const newType = this.normalizeType(column);
                const isNullable = column.isNullable === true ? "YES" : "NO";
                let columnType = dbData.data_type.toLowerCase();
                if (dbData.character_maximum_length) {
                    columnType += "(" + dbData.character_maximum_length + ")";
                }
                // const hasDbColumnAutoIncrement = dbData.EXTRA.indexOf("auto_increment") !== -1;
                // const hasDbColumnPrimaryIndex = dbData.column_key.indexOf("PRI") !== -1;
                return columnType !== newType ||
                    // dbData.COLUMN_COMMENT !== column.comment ||
                    dbData.is_nullable !== isNullable; // ||
                // hasDbColumnAutoIncrement !== column.isGenerated ||
                // hasDbColumnPrimaryIndex !== column.isPrimary;
            }).map(column => {
                // const dbData = results.find(result => result.column_name === column.name);
                // const hasDbColumnPrimaryIndex = dbData.column_key.indexOf("PRI") !== -1;
                return { columnName: column.name /*, hasPrimaryKey: hasDbColumnPrimaryIndex*/ };
            });
        });
    }
    checkIfTableExist(tableName) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_CATALOG = '${this.driver.db}' AND TABLE_NAME = '${tableName}'`;
        return this.query(sql).then(results => !!(results && results.length));
    }
    addColumnQuery(tableName, column) {
        const sql = `ALTER TABLE ${tableName} ADD ${this.buildCreateColumnSql(column, false)}`;
        return this.query(sql).then(() => { });
    }
    dropColumnQuery(tableName, columnName) {
        const sql = `ALTER TABLE ${tableName} DROP ${columnName}`;
        return this.query(sql).then(() => { });
    }
    addForeignKeyQuery(foreignKey) {
        let sql = `ALTER TABLE ${foreignKey.tableName} ADD CONSTRAINT ${foreignKey.name} ` +
            `FOREIGN KEY (${foreignKey.columnNames.join(", ")}) ` +
            `REFERENCES ${foreignKey.referencedTable.name}(${foreignKey.referencedColumnNames.join(",")})`;
        if (foreignKey.onDelete)
            sql += " ON DELETE " + foreignKey.onDelete;
        return this.query(sql).then(() => { });
    }
    dropForeignKeyQuery(tableNameOrForeignKey, foreignKeyName) {
        let tableName = tableNameOrForeignKey;
        if (tableNameOrForeignKey instanceof ForeignKeyMetadata_1.ForeignKeyMetadata) {
            tableName = tableNameOrForeignKey.tableName;
            foreignKeyName = tableNameOrForeignKey.name;
        }
        const sql = `ALTER TABLE ${tableName} DROP FOREIGN KEY \`${foreignKeyName}\``;
        return this.query(sql).then(() => { });
    }
    getTableForeignQuery(tableName) {
        const sql = `SELECT tc.constraint_name FROM information_schema.table_constraints AS tc ` +
            `WHERE constraint_type = 'FOREIGN KEY' AND tc.table_catalog='${this.driver.db}' AND tc.table_name='${tableName}';`;
        // const sql = `SELECT * FROM INFORMATION_SCHEMA.table_constraints WHERE TABLE_CATALOG = '${this.driver.db}' `
        //     + `AND TABLE_NAME = '${tableName}' AND CONSTRAINT_TYPE='FOREIGN KEY'`;
        return this.query(sql).then(results => results.map(result => result.constraint_name));
    }
    getTableUniqueKeysQuery(tableName) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_CATALOG = '${this.driver.db}' ` +
            `AND TABLE_NAME = '${tableName}' AND CONSTRAINT_TYPE = 'UNIQUE'`;
        return this.query(sql).then(results => results.map(result => result.CONSTRAINT_NAME));
    }
    getTableIndicesQuery(tableName) {
        const sql = `select
    t.relname as table_name,
    i.relname as index_name,
    a.attname as column_name,
    ix.indisprimary as is_primary
from
    pg_class t,
    pg_class i,
    pg_index ix,
    pg_attribute a
where
    t.oid = ix.indrelid
    and i.oid = ix.indexrelid
    and a.attrelid = t.oid
    and a.attnum = ANY(ix.indkey)
    and t.relkind = 'r'
    and t.relname = '${tableName}'
order by
    t.relname,
    i.relname;
`;
        return this.query(sql).then(results => {
            // exclude foreign keys
            return this.getTableForeignQuery(tableName).then(foreignKeys => {
                return results
                    .filter(result => result.is_primary !== true && foreignKeys.indexOf(result.index_name) === -1)
                    .map(result => ({
                    key: result.index_name,
                    sequence: result.Seq_in_index,
                    column: result.column_name
                }));
            });
        });
    }
    getPrimaryConstraintName(tableName) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_CATALOG = '${this.driver.db}'`
            + ` AND TABLE_NAME = '${tableName}' AND CONSTRAINT_TYPE = 'PRIMARY KEY'`;
        return this.query(sql).then(results => results && results.length ? results[0].CONSTRAINT_NAME : undefined);
    }
    dropIndex(tableName, indexName, isGenerated = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (isGenerated) {
                yield this.query(`ALTER SEQUENCE foo_id_seq OWNED BY NONE`);
            }
            const sql = `ALTER TABLE ${tableName} DROP CONSTRAINT ${indexName}`;
            yield this.query(sql);
        });
    }
    createIndex(tableName, index) {
        const sql = `CREATE ${index.isUnique ? "UNIQUE" : ""} INDEX \`${index.name}\` ON ${tableName}(${index.columns.join(", ")})`;
        return this.query(sql).then(() => { });
    }
    addUniqueKey(tableName, columnName, keyName) {
        const sql = `ALTER TABLE ${tableName} ADD CONSTRAINT ${keyName} UNIQUE (${columnName})`;
        return this.query(sql).then(() => { });
    }
    getTableColumns(tableName) {
        const sql = `SELECT column_name FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_CATALOG = '${this.driver.db}'` +
            ` AND TABLE_NAME = '${tableName}'`;
        return this.query(sql).then(results => results.map(result => result.column_name));
    }
    changeColumnQuery(tableName, columnName, newColumn, skipPrimary = false) {
        const sql = `ALTER TABLE ${tableName} CHANGE ${columnName} ${this.buildCreateColumnSql(newColumn, skipPrimary)}`;
        return this.query(sql).then(() => { });
    }
    createTableQuery(table, columns) {
        const columnDefinitions = columns.map(column => this.buildCreateColumnSql(column, false)).join(", ");
        const sql = `CREATE TABLE "${table.name}" (${columnDefinitions})`;
        return this.query(sql).then(() => { });
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    query(sql) {
        return this.driver.query(this.dbConnection, sql);
    }
    buildCreateColumnSql(column, skipPrimary) {
        let c = column.name;
        if (column.isGenerated === true)
            c += " SERIAL";
        if (!column.isGenerated)
            c += " " + this.normalizeType(column);
        if (column.isNullable !== true)
            c += " NOT NULL";
        if (column.isPrimary === true && !skipPrimary)
            c += " PRIMARY KEY";
        // TODO: implement auto increment
        if (column.comment)
            c += " COMMENT '" + column.comment + "'";
        if (column.columnDefinition)
            c += " " + column.columnDefinition;
        return c;
    }
    normalizeType(column) {
        let realType = "";
        if (typeof column.type === "string") {
            realType = column.type.toLowerCase();
        }
        else if (typeof column.type === "object" && column.type.name && typeof column.type.name === "string") {
            realType = column.type.toLowerCase();
        }
        switch (realType) {
            case "string":
                return "character varying(" + (column.length ? column.length : 255) + ")";
            case "text":
                return "text";
            case "boolean":
                return "boolean";
            case "integer":
            case "int":
                return "integer";
            case "smallint":
                return "smallint";
            case "bigint":
                return "bigint";
            case "float":
                return "real";
            case "double":
            case "number":
                return "double precision";
            case "decimal":
                if (column.precision && column.scale) {
                    return `decimal(${column.precision},${column.scale})`;
                }
                else if (column.scale) {
                    return `decimal(${column.scale})`;
                }
                else if (column.precision) {
                    return `decimal(${column.precision})`;
                }
                else {
                    return "decimal";
                }
            case "date":
                return "date";
            case "time":
                return "time";
            case "datetime":
                return "timestamp";
            case "json":
                return "text";
            case "simple_array":
                return column.length ? "character varying(" + column.length + ")" : "text";
        }
        throw new Error("Specified type (" + column.type + ") is not supported by current driver.");
    }
}
exports.PostgresSchemaBuilder = PostgresSchemaBuilder;

//# sourceMappingURL=PostgresSchemaBuilder.js.map
